<?php
namespace Service;

use Zeedhi\Framework\Session\Session;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use \Service\Structure;
use \Service\Manager\Mongo\Manager;
use \Util\Environment;
use \Util\Utilities;

class UserStructures {

    protected $dataSourceName = 'userStructures';
    /** @var \Service\Hierarchy $hierarchyService */
    protected $hierarchyService;
    /** @var \Service\Structure $structureService */
    protected $structureService;
    /** @var \Service\Environment $environment */
    protected $environment;
    /** @var \Service\Manager\Mongo\Manager $organizationManager */
    protected $organizationManager;

    public function __construct(Manager $organizationManager, Structure $structureService, Environment $environment) {
        $this->structureService = $structureService;
        $this->organizationManager = $organizationManager;
        $this->environment = $environment;
    }

     public function setHierarchyService(Hierarchy $hierarchyService) {
        $this->hierarchyService = $hierarchyService;
    }

    protected function findStructuresDetails($structuresId, $structureTypes = null) {
        if(Utilities::isTeknisa()){
            $filter = array();
        } else {
            $filter = array(
                array(
                    "columnName"=>"_id", 
                    "value"     => $structuresId,  
                    "operator"  => FilterCriteria::IN
                ));
        }
        if(!empty($structureTypes)){
            $filter[] = [
                "columnName"=>"type",
                "value"     => $structureTypes,  
                "operator"  => FilterCriteria::IN
            ];
        }
        $filterCriteria = new FilterCriteria('structure', $filter, null, 10000);
        return $this->structureService->findBy($filterCriteria);
    }

	public function getLoggedUserStructures($structureTypes = null) {
        return $this->findStructuresDetails($this->hierarchyService->getUserParentHierarchysId($this->environment->getUserId(),$structureTypes),$structureTypes);
    }

    public function findBy(FilterCriteria $filterCriteria) {
        return $this->organizationManager->findBy($filterCriteria)->getRows();
    }

    public function getUserStructuresByUserId($userId) {
        $filterCriteria = new FilterCriteria($this->dataSourceName, array(
            array(
                "columnName"=>"user",
                "value"     => $userId,
                "operator"  => "=")
        ), null, 10000
        );
        
        $userStructures = $this->findBy($filterCriteria);
        $userStructuresId = array();
        foreach ($userStructures as $structure) {
            $userStructuresId[] = $structure['structure'];
        }
        return $this->findStructuresDetails($userStructuresId);
    }

    public function persistRelations(DataSet $structureDataSet) {
        $structures = $structureDataSet->getRows();
        foreach ($structures as $structure) {
            $this->deleteOldUserStructure($structure['_id']);
            
            $dataSet = array();
            if(isset($structure['usersResponsible']) && $structure['usersResponsible']) {
                foreach ($structure['usersResponsible'] as $userId) {
                    $dataSet[] = array(
                        'structure' => $structure['_id'],
                        'user' => $userId
                    );
                }
            }
            $dataSet = new DataSet('userStructures', $dataSet);
            $this->organizationManager->persist($dataSet);
        }
    }
    
    protected function deleteOldUserStructure($structureId){
        $filterCriteria = new FilterCriteria($this->dataSourceName, array(
        	array(
        		"columnName" => "structure",
        		"operator"  => "=",
        		"value"     => $structureId
        	)
        ));
        
        $userStructures = $this->findBy($filterCriteria);
        $dataSet = new DataSet($this->dataSourceName, $userStructures);
        $this->organizationManager->delete($dataSet);

    }
    protected function findStructuresDetailsAccessProfile($aclProfiles) {
        $structures = array();
        if(!empty($aclProfiles)){
            foreach($aclProfiles as $aclProfile)
            {
                $filterCriteria = new FilterCriteria('structure', array(
                    array(
                        "columnName"=>"accessProfiles",
                        "value"     => [$aclProfile],
                        "operator"  => FilterCriteria::IN
                    )), null, 10000
                );
                $projects = $this->structureService->findBy($filterCriteria);
                if($projects){
                    foreach($projects as $project){
                        $userStructuresId[] = $project['_id'];
                        array_push($structures,$this->findStructuresDetails($userStructuresId));
                    }
                }
            }
        }
        return $structures;
    }

    public function getLoggedUserStructuresAccessProfile() {
        return $this->findStructuresDetailsAccessProfile($this->environment->getAclProfiles());
    }
}