import {ChartViewService} from "@/Services/ChartViewService";
import {Utilities} from "@/Services/Utilities";
import { ZmartChartsApex } from "@/Services/Chats/ZmartChartsApex";
import { FilterModalController } from "@/controllers/FilterModalController";
import { Filter } from '@/Services/Filter/Filter';

export class ChartViewAbstract {
    protected factMaskFunction: any;
    protected dimensionMaskFunction: any;
    protected zmartCharts: any;
    protected viewData: any;
    public period: any = {};

    public hasMultipleDimensions: Boolean = false;
    public hasMultipleFacts: Boolean = true;

    public drillDownFilters: any = {};

    public getSaveViewParams(formExtra: any, values: any, cleanValues: any) {
        const chartSeries = [
            {
                serieColor: [null],
                serieColors: values.serieColors || null,
                serieFields: values.serieFields || null,
                serieLabels: values.serieLabels || null,
                serieLineWidth: values.serieLineWidth || null,
                serieStacked: values.serieStacked || null,
                serieLineType: values.serieLineType || null,
                serieYAxisFormat: values.serieYAxisFormat || null,
                serieType: values.serieType || null
            }

        ];
        cleanValues.chartSeries = chartSeries;
        delete cleanValues.serieColors;
        return cleanValues;
    }
    getWidget(dashboard: any, widget: any) {
        const height = widget[0].style;
        if (dashboard.label !== '' || dashboard.label !== null) {
            widget[0].options.title.text = dashboard.label;
        }
        widget[0].style = "background: " + dashboard.color + "; " + height;
        return widget;
    }

    getWidgetDashboard(widget: any, dashboard: any) {
        return widget;
    }

    getZmartCharts() :ZmartChartsApex {
        if (!this.zmartCharts) {
            this.zmartCharts = new ZmartChartsApex();
        }
        return <ZmartChartsApex> this.zmartCharts;
    }

    public setDrill(charts :any, zmartCharts: ZmartChartsApex) {
        this.zmartCharts = zmartCharts;
        return charts;
    }

    getEditData(viewData: any) {
        return viewData;
    }
    setEditData(viewData: any) {
        this.viewData = viewData;
    }
    getCharts(charts: any) {
        return charts;
    }

    public async getFilterModalParameters() {
        const chart = this.zmartCharts.chartMetadata[0];
        let period: any = null;
        if (this.period) {
            if (Array.isArray(this.period)) {
                period = [];
                this.period.forEach((p: any) => {
                    period.push(p.start + " - " + p.end);
                });
            } else {
                period = [this.period.start + " - " + this.period.end];
            }
        }
        const structures = await Filter.filterGetStructureByUser();
        const dimensions = this.getAllDimensions(chart.widget.columns);
        return {
            dimensions,
            selectedDimension: chart.widget[chart.widget.dimensionProperty],
            facts: this.getAllFacts(chart.widget.columns),
            selectedFacts: this.getSelectedFacts(chart.widget.factProperties, chart.widget),
            hasMultipleDimensions: this.hasMultipleDimensions,
            hasMultipleFacts: this.hasMultipleFacts,
            drillDownFilters: this.drillDownFilters,
            structures: structures.data.dataset.getStructureByUser || [],
            selectedStructure: chart.widget.parameters.structureId[0] || undefined,
            dimensionFilterValues: this.getDimensionFilterValues(this.zmartCharts.charts[0].widget.dataSource.data, dimensions),
            period
        };
    }

    public getDimensionFilterValues(dataSourceData: any, dimensions: any) {
        if (!dataSourceData) {
            return [];
        }
        const dimensionFilter = dimensions.map((dimension: any) => {
            const uniqueValues = [...new Set(dataSourceData.map((item: any) => item[dimension.id]))];
            if (uniqueValues === undefined || uniqueValues[0] === undefined) {
                return null;
            }
            const ret: any = {};
            ret[dimension.id] = uniqueValues;
            return ret;
        });
        return Object.assign({}, ...dimensionFilter.filter((item: any) => item !== null));
    }

    public getSelectedFacts(factProperties: any, widget: any) {
        if (!Array.isArray(factProperties)) {
            return widget[factProperties];
        }
        if (factProperties.length && factProperties.length === 1) {
            return widget[factProperties[0]];
        }
        const ret: any = [];
        factProperties.forEach((fact: any) => {
            ret.push(widget[fact]);
        });
        return ret.flat();
    }

    public getAllFacts(columns: any) {
        const facts = columns.filter((column: any) => column.isFact);
        const ret: any = [];
        facts.forEach((fact: any) => {
            ret.push({
                "id": fact.name,
                "name": fact.label,
            });
        });
        return ret;
    }

    public getAllDimensions(columns: any) {
        const dimensions = columns.filter((column: any) => !column.isFact);
        const ret: any = [];
        dimensions.forEach((dimension: any) => {
            ret.push({
                "id": dimension.name,
                "name": dimension.label,
            });
        });
        return ret;
    }

    protected evalCondition(condition: String, variables: any) {
        condition = condition.replaceAll('row.', '');
        const operators: any = {
            '==': (a: any, b: any) => a === b,
            '!=': (a: any, b: any) => a !== b,
            '<': (a: any, b: any) => a < b,
            '>': (a: any, b: any) => a > b,
            '<=': (a: any, b: any) => a <= b,
            '>=': (a: any, b: any) => a >= b,
        };

        const conditions = condition.split(/&&|\|\|/);
        const conditionComparators: any = condition.match(/(\|\||&&)/g);

        const results = conditions.map((singleCondition: any) => {
            if (singleCondition === "true") {
                return true;
            }
            if (singleCondition === "false") {
                return false;
            }
            const parts = singleCondition.match(/\S+/g);

            if (parts.length !== 3) {
                console.error("Condição inválida: ", singleCondition);
                return false;
            }

            const [leftOperand, operator, rightOperand] = parts;
            let leftValue = variables[leftOperand];
            let rightValue = variables[rightOperand];
            if (leftValue === undefined) {
                leftValue = this.getValueCondition(leftOperand, rightValue);
            }
            if (rightValue === undefined) {
                rightValue = this.getValueCondition(rightOperand, leftValue);
            }
            if (leftValue === undefined || rightValue === undefined) {
                console.error("Condition error: ", singleCondition);
                return false;
            }
            return operators[operator](leftValue, rightValue);
        });
        if (conditionComparators) {
            while (conditionComparators.length > 0) {
                const comparator = conditionComparators.shift();
                const a = results.shift();
                const b = results.shift();
                if (comparator === "||") {
                    results.unshift(a || b);
                }
                if (comparator === "&&") {
                    results.unshift(a && b);
                }
            }
        }
        return results[0];
    }

    protected getValueCondition(valueFixed: any, valueRow: any) {
        if (typeof (valueRow) === 'number') {
            valueFixed = parseFloat(valueFixed);
        }
        return valueFixed;
    }

    public async getFilterModal() {
        const filterModalParameters = await this.getFilterModalParameters();
        const filterModal = new FilterModalController();
        return filterModal.getChartViewFilterModalJson(filterModalParameters);
    }
}
