import { Http, Loader, Metadata } from '@zeedhi/core';
import { Modal, ModalService, Form, Select, DateRange, CodeEditor, DialogService, LoadingService } from '@zeedhi/common';
import { TekGrid } from "@zeedhi/teknisa-components-common";
import { CustomAlertService } from '@/Services/CustomAlertService';
import { Utilities } from '@/Services/Utilities';
import { DataSourceController } from './DataSourceController';

export class DSAPIController {
    private mainController: DataSourceController = Loader.getInstance("DataSourceController");

    private get datasourceData() {
        return this.mainController.datasourceData;
    }

    public APIAddModal!: Modal;
    public APILogModal!: Modal;
    public APILogModalData: any;
    public APIData: any;

    public componentationTest(message: string) {
        console.log(message, this);
    }

    public copyToClipboard({ component }: any) {
        const name = component.name.replace('_copy', '');
        const el: any = Metadata.getInstance(name);
        const codeEl = Metadata.getInstance<CodeEditor>('copyToClipboard');
        codeEl.staticCode = el.value;
        codeEl.copyToClipboard();
    }

    public fillAPILogDetails() {
        const form = Metadata.getInstance<Form>('apiLogForm');
        const ignoreKeys: any[] = ["FILTER_DATE", "FILTER_STRUCTURE"];
        Object.keys(form.value).filter((key: string) => !ignoreKeys.includes(key)).forEach((key: any) => { form.value[key] = this.APILogModalData[key]; });
        const period = this.APILogModalData?.FILTER_DATE || {};
        form.value.FILTER_DATE = [period.START, period.END];
        form.value.FILTER_STRUCTURE = JSON.stringify(this.APILogModalData?.FILTER_STRUCTURE || {});
    }

    public async openAPIAddModal() {
        this.APIAddModal?.destroy();
        this.APIAddModal = await ModalService.createFromJson('dataSource/API/AddModal', true);
        this.APIData = {CDOPERADOR: (await Utilities.getLoggedUser())?.userId};
        this.APIAddModal.show();
    }

    public closeAPIAddModal() {
        this.APIAddModal.hide();
    }

    public async openAPILogModal({ row }: any) {
        this.APILogModal?.destroy();
        this.APILogModal = await ModalService.createFromJson('dataSource/API/APILogDetails', true);
        this.APILogModalData = row || {};
        this.APILogModal.show();
    }

    public closeAPILogModal() {
        this.APILogModal.hide();
    }

    public fillAPILogModal() {
        const form = Metadata.getInstance<Form>('apiLogForm');
        form.value = this.APILogModalData;
        console.log(form.value, this.APILogModalData);
    }

    public async openAPITabs({ row }: any) {
        this.APIData = row || {};
        this.mainController.showAPIDetail();
        const form = Metadata.getInstance<Form>('apiForm');
        this.fillApiDetails(form);
    }

    public get APITabsTitle() {
        return this.APIData?.name || 'API';
    }

    public apiFormMounted({ component }: any) {
        // Se o componente montado for formulário do modal
        if (component.name !== "modalAPIDetails") {
            return;
        }
        this.fillApiDetails(component);
    }

    public async fillApiDetails(form: Form) {
        const periodoEl = form.getChildInstance<DateRange>('FIELD_DATE_PERIOD');
        const dateFilter = form.getChildInstance<Select>('FIELD_DATE_NAME');

        const ignoreKeys: string[] = ["FIELD_DATE_PERIOD"];
        Object.keys(form.value).filter((key: string) => !ignoreKeys.includes(key)).forEach((key: any) => { form.value[key] = this.APIData[key]; });
        dateFilter.datasource.updateData(this.datasourceData.columns?.filter((col: any) => col.isDate) || []).then(() => {
            dateFilter.value = this.APIData.FIELD_DATE_NAME;
        });

        // Diminui o tamanho do menu de data
        const periodDropdown = periodoEl.childrenInstances[0].childrenInstances[0];
        periodDropdown.maxHeight = 190;
        await Utilities.generatePeriodHelperOptions(periodoEl, this.datasourceData.period);
        let periodValue = this.APIData?.PERIOD_CODE;
        if (!periodValue) {
            periodValue = this.APIData?.FIELD_DATE_PERIOD;
        }
        periodoEl.hint = '';
        setTimeout(function () {
            Utilities.setPeriodHelperValue(periodoEl, periodValue);
        });
        this.updateAPILogFilter();
    }

    public closeAPITabs() { // ...
        this.mainController.showDetail();
    }

    public updateAPIFilter() {
        if (Metadata.getInstances('API-grid').length === 0 || !Metadata.getInstances('API-grid')[0]) {
            return;
        }
        const dataSourceId = this.datasourceData._id;
        const APIGrid: any = Metadata.getInstance<TekGrid>("API-grid");
        APIGrid.datasource.dynamicFilter = {
            API_ID: [
              {
                operation: "EQUALS",
                relation: "AND",
                value: dataSourceId,
              }
            ]
          };
          APIGrid.datasource.get();
    }

    public reloadAPIGrid() {
        const grid = Metadata.getInstance<TekGrid>("API-grid");
        grid.reload();
    }

    public async saveAPIForm() {
        const isDetailsSave = this.mainController.isAPITabsVisible;
        const form = isDetailsSave ? Metadata.getInstance<Form>('apiForm') : Metadata.getInstance<Form>('modalAPIDetails');
        if (!form.validate()) {
            return;
        }
        const formValues = form.value;
        const periodoEl = form.getChildInstance<DateRange>('FIELD_DATE_PERIOD');
        const data = {
            ...formValues,
            API_ID: this.datasourceData._id,
            FIELD_DATE_PERIOD: {
                start: formValues.FIELD_DATE_PERIOD[0],
                end: formValues.FIELD_DATE_PERIOD[1]
            },
            PERIOD_CODE: periodoEl.helperValue,
            CDOPERADOR: formValues.CDOPERADOR.CDOPERADOR,
            EMAIL_API: formValues.CDOPERADOR.DSEMAILOPER,
            NMOPERADOR: formValues.CDOPERADOR.NMOPERADOR,
            name: formValues.FIELD_DATE_NAME.name,
            FIELD_DATE_NAME: formValues.FIELD_DATE_NAME.name,
            FIELD_DATE_LABEL: formValues.FIELD_DATE_NAME.label,
            _id: this.APIData._id
        };
        try {
            LoadingService.show();
            const res = await Http.post("/apiDataSource/save", {
                requestType: "DataSet",
                dataset: [data]
            });
            LoadingService.hide();
            CustomAlertService.saveSuccessAlert();
            if (res.data.messages?.length > 0) {
                const message = res.data.messages[0].message;
                DialogService.show({
                  name: 'warning-dialog',
                  type: 'error',
                  title: ' ',
                  text: message
                });
            }
            this.reloadAPIGrid();
            if (!isDetailsSave) {
                this.closeAPIAddModal();
            } else {
                const resValue = res.data.dataset?.apiDataSource[0] || [];
                Object.keys(resValue).forEach((key: string) => { form.value[key] = resValue[key]; });
            }
        } catch (err) {
            LoadingService.hide();
            CustomAlertService.errorAlert("Ocorreu um erro.");
            console.error(err);
        }
    }

    public deleteAPI(params: any) {
        Utilities.genericBeforeDeleteTekGrid(params, "/apiDataSource/remove", "API-grid");
    }

    public updateAPILogFilter() {
        if (Metadata.getInstances('APILog-grid').length === 0 || !Metadata.getInstances('APILog-grid')[0]) {
            return;
        }
        const APIGrid: any = Metadata.getInstance<TekGrid>("APILog-grid");
        APIGrid.datasource.dynamicFilter = {
            WEB_TOKEN: [
              {
                operation: "EQUALS",
                relation: "AND",
                value: this.APIData.WEB_TOKEN,
              }
            ]
          };
          APIGrid.datasource.get();
    }
}
